-- Cration de la base
/*
CREATE DATABASE PartitionDemo
GO
USE PartitionDemo

-- Table Factures

CREATE TABLE Factures
(NumFacture INT IDENTITY NOT NULL,
DateFacture Date NOT NULL,
NumClient INT NOT NULL,
Montant DECIMAL (9,2) NOT NULL DEFAULT (0)
)
ALTER TABLE Factures ADD CONSTRAINT PK_Factures PRIMARY KEY CLUSTERED (NumFacture)
*/

USE PartitionDemo
GO

SELECT TOP 100 * FROM dbo.Factures
SELECT COUNT(*) FROM dbo.Factures
SELECT MIN(DateFacture), MAX(DateFacture) FROM dbo.Factures

-- Partitionnement par mois
-- 1.  Ajout de groupes de fichiers et de fichiers
USE master
GO
DECLARE @NumPart INT = 1, @NumPartStr VARCHAR(2), @SQLCmd VARCHAR(MAX) = ''
WHILE @NumPart <= 12
BEGIN
SET @NumPartStr = +RIGHT('0'+CAST(@NumPart AS VARCHAR(2)),2)
SET @SQLCmd = @SQLCmd+'ALTER DATABASE PartitionDemo ADD FILEGROUP FG2018'+@NumPartStr+'; '
SET @SQLCmd = @SQLCmd+'ALTER DATABASE PartitionDemo ADD FILE (NAME = F2018'+@NumPartStr+', FILENAME = ''E:\MSSQLData\MSSQL14.SQL2017\MSSQL\DATA\F2018'+ @NumPartStr + '.ndf'') TO FILEGROUP FG2018'+@NumPartStr+'; '+CHAR(10)
SET @NumPart = @NumPart+1
END
PRINT @SQLCmd
EXEC (@SQLCmd)

USE PartitionDemo
GO
-- 2.  Fonction de partition
CREATE PARTITION FUNCTION FnPart_Factures(DATE)
AS RANGE RIGHT FOR VALUES ('20180201','20180301','20180401','20180501','20180601','20180701','20180801','20180901','20181001','20181101','20181201')
-- AS RANGE LEFT FOR VALUES ('20180131',EOMONTH('20180201'),'20180331','20180430','20180531','20180630','20180731','20180831','20180930','20181031','20181130')

-- 3.  Schma de partition
CREATE PARTITION SCHEME SchPart_Factures AS PARTITION FnPart_Factures
TO (FG201801,FG201802,FG201803,FG201804,FG201805,FG201806,FG201807,FG201808,FG201809,FG201810,FG201811,FG201812)
-- ALL TO PRIMARY

-- 4.  Modification et dplacement de la cl primaire (CLUSTERED) pour partitionner
ALTER TABLE Factures DROP CONSTRAINT PK_Factures
ALTER TABLE Factures ADD CONSTRAINT PK_Factures PRIMARY KEY CLUSTERED (NumFacture) ON SchPart_Factures(DateFacture) -- Erreur !
ALTER TABLE Factures ADD CONSTRAINT PK_Factures PRIMARY KEY CLUSTERED (NumFacture,DateFacture) ON SchPart_Factures(DateFacture)

-- 5.  Vrifications
SELECT	$partition.FnPart_Factures(DateFacture) AS partition,
		Count(*) AS cnt,
		Min(f.DateFacture) AS PremiereDate,MAX(f.DateFacture) AS DerniereDate
FROM	Factures f
GROUP BY $partition.FnPart_Factures(DateFacture)
ORDER BY $partition.FnPart_Factures(DateFacture)

SELECT	object_name(p.object_id) AS Nom,
		p.partition_number, 
		p.rows,
		ds.name AS groupe,
		a.type_desc as page_type_desc,
		a.total_pages AS pages
FROM sys.partitions p 
JOIN sys.allocation_units a ON p.partition_id = a.container_id
JOIN sys.data_spaces ds ON a.data_space_id = ds.data_space_id
WHERE p.object_id=object_id('Factures')

SELECT  OBJECT_NAME(i.object_id) AS TableName
       ,i.index_id
       ,i.name AS IndexName
       ,i.type_desc AS IndexType
       ,p.partition_number AS pnum
       ,a.type_desc AS page_type_desc
       ,s.name AS FileGroupName
       ,fg.is_read_only
       ,df.physical_name  -- lignes dupliques si plusieurs fichiers
       ,p.rows
       ,a.total_pages AS pages
       ,a.total_pages * 8 / 1024 AS Size_Mo
       ,p.data_compression_desc
       ,sch.name AS partition_scheme
       ,fn.name AS function_name
       ,sprv.value AS limite
       ,CASE WHEN fn.boundary_value_on_right = 1 THEN 'RIGHT'
             ELSE 'LEFT'
        END AS Side
FROM    sys.tables t
        JOIN sys.indexes i ON t.object_id = i.object_id
        JOIN sys.partitions p ON i.object_id = p.object_id
                                 AND i.index_id = p.index_id
        JOIN sys.allocation_units a ON p.partition_id = a.container_id
        JOIN sys.data_spaces s ON a.data_space_id = s.data_space_id
        JOIN sys.filegroups fg ON s.data_space_id = fg.data_space_id
        JOIN sys.database_files df ON s.data_space_id = df.data_space_id  ---- lignes dupliques si plusieurs fichiers
        LEFT JOIN sys.partition_schemes sch ON i.data_space_id = sch.data_space_id
        LEFT JOIN sys.partition_functions fn ON sch.function_id = fn.function_id
        LEFT JOIN sys.partition_range_values sprv ON fn.function_id = sprv.function_id
                                                     AND sprv.boundary_id = p.partition_number
WHERE   OBJECT_NAME(i.object_id) = 'Factures'  -- slection de la table
ORDER BY TableName
       ,i.index_id
       ,pnum;

-- 6.  Ajout d'un index
CREATE NONCLUSTERED INDEX Ix_Factures_NumClient ON Factures (NumClient) ON SchPart_Factures(DateFacture)
GO
-- Recherche avec l'index
SET STATISTICS IO ON
GO
SELECT * FROM factures WHERE NumClient = 958953
GO
USE NoPartitionDemo
GO
SELECT * FROM factures WHERE NumClient = 958953

SET STATISTICS IO OFF

-- 7.  Compression de partitions
USE PartitionDemo
GO
ALTER TABLE Factures REBUILD PARTITION = 1 WITH (DATA_COMPRESSION=PAGE)

-- Purge de donnes par SWITCH PARTITION
CREATE TABLE ExportFactures
(NumFacture INT IDENTITY NOT NULL,
DateFacture Date NOT NULL,
NumClient INT NOT NULL,
Montant DECIMAL (9,2) NOT NULL DEFAULT (0)
)
ALTER TABLE ExportFactures ADD CONSTRAINT PK_ExportFactures PRIMARY KEY CLUSTERED (NumFacture,DateFacture) WITH (DATA_COMPRESSION=PAGE) ON FG201801 
CREATE NONCLUSTERED INDEX Ix_ExportFactures_NumClient ON ExportFactures (NumClient)

ALTER TABLE Factures SWITCH PARTITION 1 TO dbo.ExportFactures

SELECT	object_name(p.object_id) AS Nom,
		p.partition_number, 
		p.rows,
		ds.name AS groupe,
		a.type_desc as page_type_desc,
		a.total_pages AS pages
FROM sys.partitions p 
JOIN sys.allocation_units a ON p.partition_id = a.container_id
JOIN sys.data_spaces ds ON a.data_space_id = ds.data_space_id
WHERE p.object_id IN (OBJECT_ID('Factures'),OBJECT_ID('ExportFactures'))
AND p.index_id = 1
ORDER BY object_name(p.object_id),p.partition_number

-- Ajout de donnes par SWITCH PARTITION
ALTER TABLE ExportFactures ADD CHECK (DateFacture <= '20180131')
ALTER TABLE ExportFactures SWITCH TO dbo.Factures PARTITION 1

SELECT	object_name(p.object_id) AS Nom,
		p.partition_number, 
		p.rows,
		ds.name AS groupe,
		a.type_desc as page_type_desc,
		a.total_pages AS pages
FROM sys.partitions p 
JOIN sys.allocation_units a ON p.partition_id = a.container_id
JOIN sys.data_spaces ds ON a.data_space_id = ds.data_space_id
WHERE p.object_id IN (OBJECT_ID('Factures'),OBJECT_ID('ExportFactures'))
AND p.index_id = 1
ORDER BY object_name(p.object_id),p.partition_number

-- Archivage par SWITCH PARTITION
CREATE TABLE ArchiveFactures
(NumFacture INT IDENTITY NOT NULL,
DateFacture Date NOT NULL,
NumClient INT NOT NULL,
Montant DECIMAL (9,2) NOT NULL DEFAULT (0)
)
ALTER TABLE ArchiveFactures ADD CONSTRAINT PK_ArchiveFactures PRIMARY KEY CLUSTERED (NumFacture,DateFacture) ON SchPart_Factures(DateFacture)
ALTER TABLE ArchiveFactures REBUILD PARTITION = 1 WITH (DATA_COMPRESSION=PAGE) 
CREATE NONCLUSTERED INDEX Ix_ArchiveFactures_NumClient ON ArchiveFactures (NumClient) ON SchPart_Factures(DateFacture)

ALTER TABLE Factures SWITCH PARTITION 1 TO ArchiveFactures PARTITION 1
ALTER TABLE Factures SWITCH PARTITION 2 TO ArchiveFactures PARTITION 2
ALTER TABLE Factures SWITCH PARTITION 3 TO ArchiveFactures PARTITION 3
ALTER TABLE Factures SWITCH PARTITION 4 TO ArchiveFactures PARTITION 4
ALTER TABLE Factures SWITCH PARTITION 5 TO ArchiveFactures PARTITION 5
ALTER TABLE Factures SWITCH PARTITION 6 TO ArchiveFactures PARTITION 6

SELECT	object_name(p.object_id) AS Nom,
		p.partition_number, 
		p.rows,
		ds.name AS groupe,
		a.type_desc as page_type_desc,
		a.total_pages AS pages
FROM sys.partitions p 
JOIN sys.allocation_units a ON p.partition_id = a.container_id
JOIN sys.data_spaces ds ON a.data_space_id = ds.data_space_id
WHERE p.object_id IN (OBJECT_ID('Factures'),OBJECT_ID('ArchiveFactures'))
AND p.index_id = 1
ORDER BY object_name(p.object_id),p.partition_number

-- Ajout d'une nouvelle partition pour 2019
ALTER DATABASE PartitionDemo ADD FILEGROUP FG201901; 
ALTER DATABASE PartitionDemo ADD FILE (NAME = F201901, FILENAME = 'E:\MSSQLData\MSSQL14.SQL2017\MSSQL\DATA\F201901.ndf') TO FILEGROUP FG201901; 
ALTER PARTITION SCHEME SchPart_Factures NEXT USED FG201901
ALTER PARTITION FUNCTION FnPart_Factures() SPLIT RANGE ('20190101')

INSERT INTO [dbo].[Factures]
           ([DateFacture]
           ,[NumClient]
           ,[Montant])
     VALUES
           ('20190101'
           ,2748
           ,495.23)

SELECT	object_name(p.object_id) AS Nom,
		p.partition_number, 
		p.rows,
		ds.name AS groupe,
		a.type_desc as page_type_desc,
		a.total_pages AS pages
FROM sys.partitions p 
JOIN sys.allocation_units a ON p.partition_id = a.container_id
JOIN sys.data_spaces ds ON a.data_space_id = ds.data_space_id
WHERE p.object_id IN (OBJECT_ID('Factures'),OBJECT_ID('ArchiveFactures'))
AND p.index_id = 1
ORDER BY object_name(p.object_id),p.partition_number


-- Mise en lecture seule de partitions
USE master
GO
ALTER DATABASE PartitionDemo SET SINGLE_USER WITH ROLLBACK IMMEDIATE
GO
ALTER DATABASE [PartitionDemo] MODIFY FILEGROUP [FG201801] READONLY
ALTER DATABASE [PartitionDemo] MODIFY FILEGROUP [FG201802] READONLY
ALTER DATABASE [PartitionDemo] MODIFY FILEGROUP [FG201803] READONLY
ALTER DATABASE [PartitionDemo] MODIFY FILEGROUP [FG201804] READONLY
ALTER DATABASE [PartitionDemo] MODIFY FILEGROUP [FG201805] READONLY
ALTER DATABASE [PartitionDemo] MODIFY FILEGROUP [FG201806] READONLY
ALTER DATABASE [PartitionDemo] MODIFY FILEGROUP [FG201807] READONLY
ALTER DATABASE [PartitionDemo] MODIFY FILEGROUP [FG201808] READONLY
ALTER DATABASE [PartitionDemo] MODIFY FILEGROUP [FG201809] READONLY
ALTER DATABASE [PartitionDemo] MODIFY FILEGROUP [FG201810] READONLY
ALTER DATABASE [PartitionDemo] MODIFY FILEGROUP [FG201811] READONLY
GO
ALTER DATABASE PartitionDemo SET MULTI_USER
GO

USE PartitionDemo
GO
SELECT * FROM Factures WHERE NumClient = 958953

UPDATE Factures SET Montant=380.00 WHERE NumClient = 958953

INSERT INTO [dbo].[Factures]
           ([DateFacture]
           ,[NumClient]
           ,[Montant])
     VALUES
           ('20190102'
           ,4132
           ,708.35)
